import type { Firestore, WriteResult } from 'firebase-admin/firestore';
import type { ReaderTaskEither } from 'fp-ts/ReaderTaskEither';
import * as t from 'io-ts';
export declare const codecs: {
    blogs: {
        idCodec: t.BrandC<t.BrandC<import("io-ts-types").NonEmptyStringC, import("./entity").EntityIdBrand>, import("./blogs").BlogIdBrand>;
        valueCodec: t.TypeC<{
            aboutArtist: t.StringC;
            artist: t.StringC;
            artistPicSmall: import("io-ts-types").OptionFromNullableC<t.StringC>;
            artistPicURL: import("io-ts-types").OptionFromNullableC<t.StringC>;
            beneficiaryAccepted: import("io-ts-types").OptionFromNullableC<t.BooleanC>;
            communityPage: import("io-ts-types").OptionFromNullableC<t.BooleanC>;
            coverPicSmall: import("io-ts-types").OptionFromNullableC<t.StringC>;
            coverPicURL: import("io-ts-types").OptionFromNullableC<t.StringC>;
            coverPicURLSmall: import("io-ts-types").OptionFromNullableC<t.StringC>;
            coverResolutions: t.RecordC<t.StringC, import("io-ts-types").OptionFromNullableC<t.StringC>>;
            coverSmall: import("io-ts-types").OptionFromNullableC<t.StringC>;
            curatorAlpha: import("io-ts-types").OptionFromNullableC<t.BooleanC>;
            createdAllowReminder: import("io-ts-types").OptionFromNullableC<t.BooleanC>;
            customlink: import("io-ts-types").OptionFromNullableC<t.StringC>;
            description: import("io-ts-types").OptionFromNullableC<t.StringC>;
            emailNewSubscribers: import("io-ts-types").OptionFromNullableC<t.BooleanC>;
            emailSettings: t.RecordC<t.StringC, import("io-ts-types").OptionFromNullableC<t.BooleanC>>;
            featuredArtists: t.ArrayC<t.StringC>;
            founder: import("io-ts-types").OptionFromNullableC<t.BooleanC>;
            freetier: t.BooleanC;
            frequency: import("io-ts-types").OptionFromNullableC<t.StringC>;
            launchDraft: import("io-ts-types").OptionFromNullableC<t.StringC>;
            launched: t.BooleanC;
            linkTitles: t.ArrayC<t.StringC>;
            links: t.ArrayC<t.StringC>;
            paypal: import("io-ts-types").OptionFromNullableC<t.BooleanC>;
            'paypal_plan_2.5': import("io-ts-types").OptionFromNullableC<t.StringC>;
            paypal_plan_5: import("io-ts-types").OptionFromNullableC<t.StringC>;
            paypal_plan_10: import("io-ts-types").OptionFromNullableC<t.StringC>;
            paypal_product_id: import("io-ts-types").OptionFromNullableC<t.StringC>;
            posts: t.ArrayC<t.StringC>;
            price: import("io-ts-types").OptionFromNullableC<t.NumberC>;
            profileResolutions: import("io-ts-types").OptionFromNullableC<t.UnknownRecordC>;
            reminderLaunchSent: import("io-ts-types").OptionFromNullableC<t.BooleanC>;
            reverseTitle: import("io-ts-types").OptionFromNullableC<t.BooleanC>;
            scheduled: import("io-ts-types").OptionFromNullableC<t.BooleanC>;
            show: import("io-ts-types").OptionFromNullableC<t.StringC>;
            stripeProductID: import("io-ts-types").OptionFromNullableC<t.StringC>;
            subscribers: t.ArrayC<t.BrandC<t.BrandC<import("io-ts-types").NonEmptyStringC, import("./entity").EntityIdBrand>, import("./users").UserIdBrand>>;
            tipButton: import("io-ts-types").OptionFromNullableC<t.BooleanC>;
            title: t.StringC;
            verified: t.BooleanC;
        }>;
    };
    music: {
        idCodec: t.BrandC<t.BrandC<import("io-ts-types").NonEmptyStringC, import("./entity").EntityIdBrand>, import("./music").MusicIdBrand>;
        valueCodec: t.TypeC<{
            blogId: t.BrandC<t.BrandC<import("io-ts-types").NonEmptyStringC, import("./entity").EntityIdBrand>, import("./blogs").BlogIdBrand>;
            title: import("io-ts-types").NonEmptyStringC;
            publicUrl: import("io-ts-types").NonEmptyStringC;
            fileId: import("io-ts-types").NonEmptyStringC;
            duration: t.NumberC;
            imageUrl: import("io-ts-types").OptionFromNullableC<import("io-ts-types").NonEmptyStringC>;
        }>;
    };
};
export type CollectionKey = keyof typeof codecs;
export type IdOf<K extends CollectionKey> = t.TypeOf<(typeof codecs)[K]['idCodec']>;
export type ValueOf<K extends CollectionKey> = t.TypeOf<(typeof codecs)[K]['valueCodec']>;
export type Entity<K extends CollectionKey> = {
    id: IdOf<K>;
} & ValueOf<K>;
export type FirestoreError = {
    _tag: 'DecodeError';
    error: t.Errors;
} | {
    _tag: 'InsertError';
    error: unknown;
} | {
    _tag: 'UpdateError';
    error: unknown;
} | {
    _tag: 'DeleteError';
    error: unknown;
} | {
    _tag: 'ReadError';
    error: unknown;
};
export declare const DecodeError: (error: t.Errors) => FirestoreError;
export declare const InsertError: (error: unknown) => FirestoreError;
export declare const UpdateError: (error: unknown) => FirestoreError;
export declare const DeleteError: (error: unknown) => FirestoreError;
export declare const ReadError: (error: unknown) => FirestoreError;
export declare const create: <K extends "blogs" | "music">(collectionKey: K) => (value: ValueOf<K>) => ReaderTaskEither<Firestore, FirestoreError, IdOf<K>>;
export declare const read: <K extends "blogs" | "music">(collectionKey: K) => (id: IdOf<K>) => ReaderTaskEither<Firestore, FirestoreError, Entity<K>>;
export declare const update: <K extends "blogs" | "music">(collectionKey: K) => (id: IdOf<K>) => (value: ValueOf<K>) => ReaderTaskEither<Firestore, FirestoreError, WriteResult>;
export declare const remove: <K extends "blogs" | "music">(collectionKey: K) => (id: IdOf<K>) => ReaderTaskEither<Firestore, FirestoreError, WriteResult>;
