"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.remove = exports.update = exports.read = exports.create = exports.ReadError = exports.DeleteError = exports.UpdateError = exports.InsertError = exports.DecodeError = exports.codecs = void 0;
var E = require("fp-ts/Either");
var RTE = require("fp-ts/ReaderTaskEither");
var TE = require("fp-ts/TaskEither");
var function_1 = require("fp-ts/function");
var blogs_1 = require("./blogs");
var music_1 = require("./music");
exports.codecs = {
    blogs: { idCodec: blogs_1.blogIdCodec, valueCodec: blogs_1.blogCodec },
    music: { idCodec: music_1.musicIdCodec, valueCodec: music_1.musicCodec },
};
var DecodeError = function (error) { return ({
    _tag: 'DecodeError',
    error: error,
}); };
exports.DecodeError = DecodeError;
var InsertError = function (error) { return ({
    _tag: 'InsertError',
    error: error,
}); };
exports.InsertError = InsertError;
var UpdateError = function (error) { return ({
    _tag: 'UpdateError',
    error: error,
}); };
exports.UpdateError = UpdateError;
var DeleteError = function (error) { return ({
    _tag: 'DeleteError',
    error: error,
}); };
exports.DeleteError = DeleteError;
var ReadError = function (error) { return ({
    _tag: 'ReadError',
    error: error,
}); };
exports.ReadError = ReadError;
// type IdDecoder<K extends CollectionKey> = (i: unknown) => t.Validation<IdOf<K>>;
var decodeId = function (collection) {
    return function (id) {
        var r = exports.codecs[collection].idCodec.decode(id);
        if (E.isLeft(r)) {
            return E.left((0, exports.DecodeError)(r.left));
        }
        else {
            return E.right(r.right);
        }
        // return pipe(
        //   id,
        //   codecs[collection].idCodec.decode as IdDecoder<K>,
        //   E.mapLeft(DecodeError),
        // );
    };
};
var decodeValue = function (collection) {
    return function (value) {
        var r = exports.codecs[collection].valueCodec.decode(value);
        if (E.isLeft(r)) {
            return E.left((0, exports.DecodeError)(r.left));
        }
        else {
            return E.right(r.right);
        }
    };
};
var create = function (collectionKey) {
    return function (value) {
        var r = (0, function_1.pipe)(RTE.ask(), RTE.chainIOK(function (firestore) { return function () { return firestore.collection(collectionKey); }; }), RTE.chainTaskEitherK(function (collection) { return TE.tryCatch(function () { return collection.add(value); }, exports.InsertError); }), RTE.chainEitherK(function (doc) { return decodeId(collectionKey)(doc.id); }));
        return r;
    };
};
exports.create = create;
var read = function (collectionKey) {
    return function (id) {
        var r = (0, function_1.pipe)(RTE.ask(), RTE.chainIOK(function (firestore) { return function () { return firestore.collection(collectionKey); }; }), RTE.chainIOK(function (collection) { return function () { return collection.doc(id); }; }), RTE.chainTaskEitherK(function (doc) { return TE.tryCatch(function () { return doc.get(); }, exports.ReadError); }), RTE.chainIOK(function (snap) { return function () { return snap.data(); }; }), RTE.chainEitherK(decodeValue(collectionKey)), RTE.map(function (value) { return (__assign({ id: id }, value)); }));
        return r;
    };
};
exports.read = read;
var update = function (collectionKey) {
    return function (id) {
        return function (value) {
            var r = (0, function_1.pipe)(RTE.ask(), RTE.chainIOK(function (firestore) { return function () { return firestore.collection(collectionKey); }; }), RTE.chainIOK(function (collection) { return function () { return collection.doc(id); }; }), RTE.chainTaskEitherK(function (doc) { return TE.tryCatch(function () { return doc.update(value); }, exports.UpdateError); }));
            return r;
        };
    };
};
exports.update = update;
var remove = function (collectionKey) {
    return function (id) {
        var r = (0, function_1.pipe)(RTE.ask(), RTE.chainIOK(function (firestore) { return function () { return firestore.collection(collectionKey); }; }), RTE.chainIOK(function (collection) { return function () { return collection.doc(id); }; }), RTE.chainTaskEitherK(function (doc) { return TE.tryCatch(function () { return doc.delete(); }, exports.DeleteError); }));
        return r;
    };
};
exports.remove = remove;
