import * as t from 'io-ts';
import * as tt from 'io-ts-types';
import { EntityId, entityIdCodec } from './entity';
import { userIdCodec } from './users';

export interface BlogIdBrand {
  readonly BlogId: unique symbol;
}

export const blogIdCodec = t.brand(entityIdCodec, (s): s is t.Branded<EntityId, BlogIdBrand> => true, 'BlogId');
export type BlogId = t.TypeOf<typeof blogIdCodec>;

export const blogCodec = t.type({
  aboutArtist: t.string,
  artist: t.string,
  artistPicSmall: tt.optionFromNullable(t.string),
  artistPicURL: tt.optionFromNullable(t.string),
  beneficiaryAccepted: tt.optionFromNullable(t.boolean),
  communityPage: tt.optionFromNullable(t.boolean),
  coverPicSmall: tt.optionFromNullable(t.string),
  coverPicURL: tt.optionFromNullable(t.string),
  coverPicURLSmall: tt.optionFromNullable(t.string),
  coverResolutions: t.record(t.string, tt.optionFromNullable(t.string)),
  coverSmall: tt.optionFromNullable(t.string),
  curatorAlpha: tt.optionFromNullable(t.boolean),
  createdAllowReminder: tt.optionFromNullable(t.boolean),
  customlink: tt.optionFromNullable(t.string),
  description: tt.optionFromNullable(t.string),
  emailNewSubscribers: tt.optionFromNullable(t.boolean),
  emailSettings: t.record(t.string, tt.optionFromNullable(t.boolean)),
  featuredArtists: t.array(t.string),
  founder: tt.optionFromNullable(t.boolean),
  freetier: t.boolean,
  frequency: tt.optionFromNullable(t.string),
  launchDraft: tt.optionFromNullable(t.string),
  launched: t.boolean,
  linkTitles: t.array(t.string),
  links: t.array(t.string),
  paypal: tt.optionFromNullable(t.boolean),
  'paypal_plan_2.5': tt.optionFromNullable(t.string),
  paypal_plan_5: tt.optionFromNullable(t.string),
  paypal_plan_10: tt.optionFromNullable(t.string),
  paypal_product_id: tt.optionFromNullable(t.string),
  posts: t.array(t.string),
  price: tt.optionFromNullable(t.number),
  profileResolutions: tt.optionFromNullable(t.UnknownRecord),
  reminderLaunchSent: tt.optionFromNullable(t.boolean),
  reverseTitle: tt.optionFromNullable(t.boolean),
  scheduled: tt.optionFromNullable(t.boolean),
  show: tt.optionFromNullable(t.string),
  stripeProductID: tt.optionFromNullable(t.string),
  subscribers: t.array(userIdCodec),
  tipButton: tt.optionFromNullable(t.boolean),
  title: t.string,
  verified: t.boolean,
});
